Package: HiBED
Type: Package
Title: HiBED
Version: 1.10.0
Authors@R: c(person("Ze", "Zhang", role = c("cre", "aut"),
            email = "ze.zhang.gr@dartmouth.edu", comment=c(ORCID="0000-0001-9854-5823")),
            person(c("Lucas", "A."), "Salas", role = c("aut"),
            email = "lucas.a.salas.diaz@dartmouth.edu"))
Description: Hierarchical deconvolution for extensive cell type resolution in the human brain 
             using DNA methylation. The HiBED deconvolution estimates proportions up to 7 cell 
             types (GABAergic neurons, glutamatergic neurons, astrocytes, microglial cells, 
			 oligodendrocytes, endothelial cells, and stromal cells) in bulk brain tissues. 
License: GPL-3
Encoding: UTF-8
LazyData: false
VignetteBuilder: knitr
biocViews: ExperimentData, Homo_sapiens_Data, Tissue, MicroarrayData,
        Genome, MethylationArrayData, PackageTypeData
RoxygenNote: 7.2.3
URL: https://github.com/SalasLab/HiBED
Imports: dplyr, FlowSorted.Blood.EPIC, tibble, FlowSorted.DLPFC.450k,
        minfi, utils, AnnotationHub, SummarizedExperiment
Suggests: knitr, rmarkdown, testthat,
        IlluminaHumanMethylation450kmanifest
BugReports: https://github.com/SalasLab/HiBED/issues.
git_url: https://git.bioconductor.org/packages/HiBED
git_branch: RELEASE_3_23
git_last_commit: 2bb65e4
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-05-05
NeedsCompilation: no
Packaged: 2026-05-05 15:25:31 UTC; biocbuild
Author: Ze Zhang [cre, aut] (ORCID: <https://orcid.org/0000-0001-9854-5823>),
  Lucas A. Salas [aut]
Maintainer: Ze Zhang <ze.zhang.gr@dartmouth.edu>
Depends: R (>= 3.5.0)

# Generated by roxygen2: do not edit by hand

export(HiBED_deconvolution)
import(AnnotationHub)
import(FlowSorted.Blood.EPIC)
import(FlowSorted.DLPFC.450k)
import(SummarizedExperiment)
import(dplyr)
import(tibble)
importFrom(minfi,getBeta)
importFrom(minfi,preprocessRaw)
importFrom(utils,data)

# HiBED
### Publication DOI: [10.3389/fnins.2023.1198243](https://www.frontiersin.org/articles/10.3389/fnins.2023.1198243/full)
### Bioconductor Package: https://bioconductor.org/packages/devel/data/experiment/html/HiBED.html
Hierarchical deconvolution for extensive cell type resolution in the human brain using DNA methylation.
The HiBED deconvolution estimates proportions up to 7 cell types (GABAergic neurons, glutamatergic neurons, astrocytes, microglial cells, oligodendrocytes, endothelial cells, and stromal cells) in bulk brain tissues.

![Figure1](https://user-images.githubusercontent.com/32206453/224516354-75e2b4bd-102f-4c11-be84-e40f36daf5f0.png)

## Installation
```
devtools::install_github("SalasLab/HiBED")
```

## Load package
```
library(HiBED)
```

## Deconvolution function
```
?HiBED_deconvolution
```

## Example
```
library(FlowSorted.Blood.EPIC)
library(FlowSorted.DLPFC.450k)
library(minfi)
Mset<-preprocessRaw(FlowSorted.DLPFC.450k)
Examples_Betas<-getBeta(Mset)
HiBED_result<-HiBED_deconvolution(Examples_Betas, h=2)
head(HiBED_result)
```

## EPICv2 Solution
#### if you're having trouble dowloading v2 annotation files, you can find them at 
####  https://www.dropbox.com/sh/rbxjhq9zalqq58e/AAABR8kKegXKVMeNJV8a2lJRa?dl=0
```
library(HiBED)
library(minfi)
library(IlluminaHumanMethylationEPICv2anno.20a1.hg38)
library(IlluminaHumanMethylationEPICv2manifest)
```
#### dir should be the folder containing EPICv2 IDATs
```
v2_RGset = read.metharray.exp("dir",recursive = TRUE) 
annotation(v2_RGset)["array"] = "IlluminaHumanMethylationEPICv2" #Update annotation files for v2
annotation(v2_RGset)["annotation"] = "20a1.hg38"
v2_MSet <-preprocessNoob(v2_RGset)
v2_Betas<-getBeta(v2_MSet)
v2_Betas<- sesame::betasCollapseToPfx(v2_Betas)
HiBED_deconvolution(v2_Betas, h=2)
```

#' @title
#' HiBED_deconvolution
#'
#' @name
#' HiBED_deconvolution
#'
#' @description
#' The function estimates proportions up to 7 cell types in brain tissues.
#'
#' @examples
#' #Step 1: Load required libraries
#' library(FlowSorted.Blood.EPIC)
#' library(FlowSorted.DLPFC.450k)
#' #Step 2: Load example data and preprocess
#' Mset<-minfi::preprocessRaw(FlowSorted.DLPFC.450k)
#' Examples_Betas<-minfi::getBeta(Mset)
#' #Step 3: Run HiBED and show results
#' HiBED_result<-HiBED_deconvolution(Examples_Betas, h=2)
#' head(HiBED_result)
#'
#' @param
#' Beta Methylation beta in the format of matrix or data frame or Mset or
#' SummarizedExperiment from brain samples.
#'
#' @param
#' h Numeric variable.
#' Specify the layer of deconvolution in the hierarchical model. Default is 2.
#'
#' @return
#' A matrix with predicted cell proportions in brain tissues.
#'
#' @import  FlowSorted.Blood.EPIC
#'
#' @import  dplyr
#'
#' @import  tibble
#'
#' @import  FlowSorted.DLPFC.450k
#'
#' @import  AnnotationHub
#'
#' @import  SummarizedExperiment
#'
#' @importFrom minfi preprocessRaw
#'
#' @importFrom minfi getBeta
#'
#' @importFrom utils data
#'
#' @export
#'

HiBED_deconvolution <- function(Beta, h=2){
  data_env <- new.env(parent = emptyenv())
  data("HiBED_Libraries", envir = data_env, package = "HiBED")
  HiBED_Libraries<-data_env[["HiBED_Libraries"]]
  Library_Layer1<-as.data.frame(
    assay(HiBED_Libraries$Library_Layer1,"counts"))
  Library_Layer2A<-as.data.frame(
    assay(HiBED_Libraries$Library_Layer2A,"counts"))
  Library_Layer2B<-as.data.frame(
    assay(HiBED_Libraries$Library_Layer2B,"counts"))
  Library_Layer2C<-as.data.frame(
    assay(HiBED_Libraries$Library_Layer2C,"counts"))

  if ((!is(h, "numeric"))) {
    stop(strwrap(sprintf(
      "object is of class '%s', but needs to be of
                                class 'numeric' to use this function",
      class(h)
    ),
    width = 80, prefix = " ",
    initial = ""
    ))
  }
  if (is(Beta, "MethylSet")) {
    Beta <- getBeta(Beta)
  }
  if ((!is(Beta, "matrix")) && (!is(Beta, "data.frame")) &&
      (!is(Beta, "SummarizedExperiment"))) {
    stop(strwrap(sprintf(
      "object is of class '%s', but needs to be of
                                class 'matrix' 'data.frame' or
                                'SummarizedExperiment' to use this function",
      class(Beta)
    ),
    width = 80, prefix = " ",
    initial = ""
    ))
  }

  if (is(Beta,'SummarizedExperiment')){
    Beta <- assay(Beta)
  }


  proj1<-as.data.frame(projectCellType_CP(Beta[rownames(Library_Layer1[
    which(rownames(Library_Layer1) %in% rownames(Beta)),]),],
                                          as.matrix(Library_Layer1[
                                            which(rownames(Library_Layer1)
                                                  %in% rownames(Beta)),]),
    lessThanOne = TRUE))
  proj1[proj1<1e-05]<-0


  proj2A<-as.data.frame(projectCellType_CP(Beta[
    rownames(Library_Layer2A[which(rownames(Library_Layer2A)
                                   %in% rownames(Beta)),]),],
                                           as.matrix(Library_Layer2A[
                                             which(rownames(Library_Layer2A)
                                                   %in% rownames(Beta)),]),
    lessThanOne = TRUE))
  proj2A[proj2A<1e-05]<-0

  for (i in seq_len(nrow(proj2A))) {
    z<-1/sum(proj2A[i,])
    proj2A[i,]<-z*proj2A[i,]
  }

  proj2B<-as.data.frame(projectCellType_CP(Beta[rownames(Library_Layer2B[
    which(rownames(Library_Layer2B) %in% rownames(Beta)),]),],
                                           as.matrix(Library_Layer2B[
                                             which(rownames(Library_Layer2B)
                                                   %in% rownames(Beta)),]),
    lessThanOne = TRUE))
  proj2B[proj2B<1e-05]<-0

  for (i in seq_len(nrow(proj2B))) {
    z<-1/sum(proj2B[i,])
    proj2B[i,]<-z*proj2B[i,]
  }


  proj2C<-as.data.frame(projectCellType_CP(Beta[rownames(Library_Layer2C[
    which(rownames(Library_Layer2C) %in% rownames(Beta)),]),],
                                           as.matrix(Library_Layer2C[
                                             which(rownames(Library_Layer2C)
                                                   %in% rownames(Beta)),]),
    lessThanOne = TRUE))
  proj2C[proj2C<1e-05]<-0

  for (i in seq_len(nrow(proj2C))) {
    z<-1/sum(proj2C[i,])
    proj2C[i,]<-z*proj2C[i,]
  }


  proj<-proj1
  h1_proj<-proj
  proj2A<-proj2A[,-which(colnames(proj2A) %in% c("Glial","Neuronal")),
                 drop=FALSE]
  proj2A<-proj2A/rowSums(proj2A)
  proj<-cbind(proj[, -which(colnames(proj) %in% c("Endothelial and Stromal"))],
              proj[, c("Endothelial and Stromal")] * proj2A)
  h2A_proj<-proj

  proj2B<-proj2B[,-which(colnames(proj2B) %in% c("Endothelial and Stromal",
                                                 "Neuronal")),
                 drop=FALSE]
  proj2B<-proj2B/rowSums(proj2B)
  proj<-cbind(proj[, -which(colnames(proj) %in% c("Glial"))],
              proj[, c("Glial")] * proj2B)
  h2B_proj<-proj

  proj2C<-proj2C[,-which(colnames(proj2C) %in% c("Endothelial and Stromal",
                                                 "Glial")),
                 drop=FALSE]
  proj2C<-proj2C/rowSums(proj2C)
  proj<-cbind(proj[, -which(colnames(proj) %in% c("Neuronal"))],
              proj[, c("Neuronal")] * proj2C)
  h2C_proj<-proj

  is.nan.data.frame <- function(x)
    do.call(cbind, lapply(x, is.nan))

  proj[is.nan.data.frame(proj)]<-0

  SumValue<-NA
  proj$SumValue<-round(rowSums(proj),2)

  proj_low<-proj %>% filter(SumValue<1)
  ID_low<-rownames(proj_low)

  proj<-proj[,!colnames(proj)=="SumValue"]

  proj[ID_low,]<- h2C_proj[ID_low,]

  proj$Neuronal<-h1_proj$Neuronal
  proj$Glial<-h1_proj$Glial
  proj$`Endothelial and Stromal`<-h1_proj$`Endothelial and Stromal`

  proja<-proj[!rownames(proj)%in%ID_low,]
  proja[is.nan.data.frame(proja)]<-0
  proj[rownames(proja),]<-proja

  if(h==1){
    output<-proj[,c("Endothelial and Stromal","Glial","Neuronal")]
  }else{output<-proj[,c("Endothelial","Stromal","Astrocyte","Microglial",
                        "Oligodendrocyte","GABA","GLU")]
  }

  output$SumValue<-rowSums(output)

  output_low<-output %>% filter(SumValue == "NaN")
  ID_low<-rownames(output_low)
  if(length(ID_low)!=0){
    message("NaN indicates noisy deconvolution signal, thus removed")}
  return(output[,!colnames(output)=="SumValue"]*100)
}

test_that("errors if bad parameters", {
  library(HiBED)
  library(FlowSorted.DLPFC.450k)
  data(FlowSorted.DLPFC.450k)
  expect_error(HiBED_deconvolution(FlowSorted.DLPFC.450k,
                                   h=2
  ))
  Mset<-minfi::preprocessRaw(FlowSorted.DLPFC.450k)
  Examples_Betas<-minfi::getBeta(Mset)
  expect_error(HiBED_deconvolution(Examples_Betas,
                                   h=2,"brain"
  ))
  expect_error(HiBED_deconvolution(Mset,
                                   h="one"
  ))
})

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(cache = FALSE, warning = FALSE, message = FALSE, 
                        cache.lazy = FALSE,collapse = TRUE, comment = "#>"
)
library(HiBED)

## ----eval=TRUE----------------------------------------------------------------
library(HiBED)

## ----eval=TRUE----------------------------------------------------------------
data("HiBED_Libraries")

## ----eval=TRUE----------------------------------------------------------------

# Step 1 load and process example
library(FlowSorted.Blood.EPIC)
library(FlowSorted.DLPFC.450k)
library(minfi)
Mset<-preprocessRaw(FlowSorted.DLPFC.450k)
             
Examples_Betas<-getBeta(Mset)


# Step 2: use the HiBED_deconvolution function in combinatation with the
# reference libraries for brain cell deconvolution.


 HiBED_result<-HiBED_deconvolution(Examples_Betas, h=2)
                                
 head(HiBED_result)



## -----------------------------------------------------------------------------
sessionInfo()

### =========================================================================
### HiBED metadata
### -------------------------------------------------------------------------
###

meta <- data.frame(
  Title = c("HiBED"),
  Description = c(paste0("The HiBED package ",
                         "contains reference libraries derived from Illumina ",
                         "HumanMethylation450K and ",
                         "Illumina HumanMethylationEPIC ",
                         "DNA methylation microarrays (Zhang Z, Salas LA ",
                         "et al. under review), consisting of 6 astrocyte, ",
                         "12 endothelial, 5 GABAergic neuron, 5 glutamatergic ",
                         "neuron, 18 microglial, 20 oligodendrocyte, and 5 ",
                         "stromal samples from public resources.")),
  BiocVersion = c("3.9"),
  Genome = rep("hg19", 1),
  SourceType = rep("tar.gz", 1),
  SourceUrl = paste0("https://bit.ly/3JygxbC, ", "https://bit.ly/3ZYMtuX, ",
                     "https://bit.ly/3mMFB5P, ", "https://bit.ly/429rSpB"
                     ),
  SourceVersion = "Mar 11 2023",
  Species = "Homo sapiens",
  TaxonomyId = 9606,
  Coordinate_1_based = TRUE,
  DataProvider = paste0("Synapse, ", "Bioconductor, ","GEO"),
  Maintainer = "Ze Zhang <ze.zhang.gr@dartmouth.edu>",
  RDataClass = c("Beta Matrix") ,
  DispatchClass = c(rep("Rda",1)),
  RDataPath = c(paste0("HiBED/",
                       "HiBED.rda")),
  Tags = "",
  Notes = paste0("Guintivano et al 2013, ","Weightman Potter PG et al 2021, ", "de Witte et al 2022, ",
                 "Mendizabal et al 2015, ", "Lin et al 2018, ", "Kozlenkov et al 2018")
)

write.csv(meta, file="inst/extdata/metadata.csv", row.names=FALSE)

#' HiBED library CpGs matrix stored in SummarizedExperiment for
#' brain tissue DNA methylation deconvolution
#'
#' @description
#'     This object contains 4 matrices of the the average
#'     DNA methylation values of the probes included in 4 layers of the HiBED
#'     deconvolution. These CpGs are used as the backbone for deconvolution and
#'     were selected because their methylation signature differs across the
#'     seven brain cell subtypes.
#'
#' @format The list contains matrices are 81 x 3, 183 x 4, 237 x 5, 120 x 4
#'
#'         The format is:
#'         num [1:81, 1:3] 0.04592944  0.02268472  0.88886150 ...
#'
#'
#' @usage
#' data("HiBED_Libraries")
#'
#' @return A list with 4 libraries in SummarizedExperiment formats
#'
#' @examples
#' data("HiBED_Libraries")
#' head(HiBED_Libraries)
"HiBED_Libraries"

---
title: "HiBED"
author: "Ze Zhang"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
    %\VignetteIndexEntry{HiBED}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(cache = FALSE, warning = FALSE, message = FALSE, 
                        cache.lazy = FALSE,collapse = TRUE, comment = "#>"
)
library(HiBED)
```

The HiBED package contains reference libraries derived from 
Illumina HumanMethylation450K and Illumina HumanMethylationEPIC DNA methylation 
microarrays (Zhang Z, Salas LA et al. 2023), consisting of 6 astrocyte,
12 endothelial, 5 GABAergic neuron, 5 glutamatergic neuron, 18 microglial, 
20 oligodendrocyte, and 5 stromal samples from public resources.

The reference libraries were used to estimate proportions of 7 major brain cell
types in 450K and EPIC bulk brain samples using a modified version of 
the algorithm constrained projection/quadratic programming described 
in Houseman et al. 2012. 


**Loading package:**  
```{r eval=TRUE}
library(HiBED)
```

**Objects included:**  
1. *HiBED_Libraries* contains 4 libraries for deconvolution  
```{r eval=TRUE}
data("HiBED_Libraries")
```

2. *HiBED_deconvolution function for brain cell deconvolution:*
 
 We offer the function HiBED_deconvolution to estimate proportions for 7 major
 brain cell types, including GABAergic neurons, glutamatergic neurons, 
 astrocytes, microglial cells, oligodendrocytes, endothelial cells, and stromal 
 cells. The estimates are calculated using modified CP/QP 
 method described in Houseman et al. 2012.    
 *see ?HiBED_deconvolution for details*  
```{r eval=TRUE}

# Step 1 load and process example
library(FlowSorted.Blood.EPIC)
library(FlowSorted.DLPFC.450k)
library(minfi)
Mset<-preprocessRaw(FlowSorted.DLPFC.450k)
             
Examples_Betas<-getBeta(Mset)


# Step 2: use the HiBED_deconvolution function in combinatation with the
# reference libraries for brain cell deconvolution.


 HiBED_result<-HiBED_deconvolution(Examples_Betas, h=2)
                                
 head(HiBED_result)


```

```{r}
sessionInfo()
```


**References**

Z Zhang, LA Salas et al. (2023) SHierarchical deconvolution for extensive cell 
type resolution in the human brain using DNA methylation. 
Under Review

J. Guintivano, et al. (2013). A cell epigenotype specific 
model for the correction of brain cellular heterogeneity bias and its 
application to age, brain region and major depression. Epigenetics, 
8(3):290–302, 2013. doi: [10.4161/epi.23924] 
(https://dx.doi.org/10.4161/epi.23924).

Weightman Potter PG, et al. (2021) Attenuated Induction of the Unfolded Protein 
Response in Adult Human Primary Astrocytes in Response to Recurrent Low Glucose. 
Front Endocrinol (Lausanne) 2021;12:671724. doi: 
[10.3389/fendo.2021.671724]
(https://dx.doi.org/10.3389/fendo.2021.671724).

Kozlenkov, et al. (2018) A unique role for DNA (hydroxy)methylation in 
epigenetic regulation of human inhibitory neurons. Sci. Adv. 2018;4:eaau6190. 
doi: [10.1126/sciadv.aau6190]
(https://dx.doi.org/10.1126/sciadv.aau6190). 

de Whitte, et al. (2022) Contribution of Age, Brain Region, Mood Disorder 
Pathology, and Interindividual Factors on the Methylome of Human Microglia. 
Biological Psychiatry March 15, 2022; 91:572–581. 
doi: [10.1016/j.biopsych.2021.10.020]
(https://doi.org/10.1016/j.biopsych.2021.10.020). 

X Lin, et al. (2018) Cell type-specific DNA methylation in neonatal cord tissue
and cord blood: A 850K-reference panel and comparison of cell-types. 
Epigenetics. 13:941–58. doi: 
[10.1080/15592294.2018.1522929]
(https://dx.doi.org/10.1080/15592294.2018.1522929).

LA Salas et al. (2022). Enhanced cell deconvolution of peripheral blood using 
DNA methylation for high-resolution immune profiling. 
Nature Communications 13(1):761. 
doi:[10.1038/s41467-021-27864-7](https://dx.doi.org/10.1038/s41467-021-27864-7).  

EA Houseman et al. (2012) DNA methylation arrays as surrogate
measures of cell mixture distribution. BMC Bioinformatics 13, 86.
doi: [10.1186/1471-2105-13-86](https://dx.doi.org/10.1186/1471-2105-13-86).  

[minfi](http://bioconductor.org/packages/release/bioc/html/minfi.html)
Tools to analyze & visualize Illumina Infinium methylation arrays.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HiBED_Libraries.R
\docType{data}
\name{HiBED_Libraries}
\alias{HiBED_Libraries}
\title{HiBED library CpGs matrix stored in SummarizedExperiment for
brain tissue DNA methylation deconvolution}
\format{
The list contains matrices are 81 x 3, 183 x 4, 237 x 5, 120 x 4

        The format is:
        num [1:81, 1:3] 0.04592944  0.02268472  0.88886150 ...
}
\usage{
data("HiBED_Libraries")
}
\value{
A list with 4 libraries in SummarizedExperiment formats
}
\description{
This object contains 4 matrices of the the average
    DNA methylation values of the probes included in 4 layers of the HiBED
    deconvolution. These CpGs are used as the backbone for deconvolution and
    were selected because their methylation signature differs across the
    seven brain cell subtypes.
}
\examples{
data("HiBED_Libraries")
head(HiBED_Libraries)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HiBED_deconvolution.R
\name{HiBED_deconvolution}
\alias{HiBED_deconvolution}
\title{HiBED_deconvolution}
\usage{
HiBED_deconvolution(Beta, h = 2)
}
\arguments{
\item{Beta}{Methylation beta in the format of matrix or data frame or Mset or
SummarizedExperiment from brain samples.}

\item{h}{Numeric variable.
Specify the layer of deconvolution in the hierarchical model. Default is 2.}
}
\value{
A matrix with predicted cell proportions in brain tissues.
}
\description{
The function estimates proportions up to 7 cell types in brain tissues.
}
\examples{
#Step 1: Load required libraries
library(FlowSorted.Blood.EPIC)
library(FlowSorted.DLPFC.450k)
#Step 2: Load example data and preprocess
Mset<-minfi::preprocessRaw(FlowSorted.DLPFC.450k)
Examples_Betas<-minfi::getBeta(Mset)
#Step 3: Run HiBED and show results
HiBED_result<-HiBED_deconvolution(Examples_Betas, h=2)
head(HiBED_result)

}
